#include "Velcro.h"


// ==============================================================
// Airfoil lift/drag functions
// ==============================================================
void VLiftCoeff(double aoa, double M, double Re, double* cl, double* cm, double* cd)
{
	static const double step = RAD * 15.0;
	static const double istep = 1.0 / step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = { 0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.02, 0.6955,    0.66,   0.42, 0.24, 0.10, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1 };
	static const double CM[nabsc] = { 0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0 };
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if (M >= 5.0) {
		Cl0 = -0.187040;
		Cl1 = 1.675600;
		Cd0 = 0.068540;
		Cd1 = -0.21767;
		Cd2 = 1.3632;
		*cl = Cl0 + Cl1 * aoa;
		*cd = Cd0 + Cd1 * aoa + Cd2 * aoa * aoa;
	}
	aoa += PI;
	int idx = max(0, min(23, (int)(aoa * istep)));
	double d = aoa * istep - idx;
	*cm = (CM[idx] + (CM[idx + 1] - CM[idx]) * d);
	if (M < 1.25) {
		*cl = CL[idx] + (CL[idx + 1] - CL[idx]) * d;
		*cd = 0.04 + oapiGetInducedDrag(*cl, 2.1, 0.82);
	}
	else if (M < 5.0) {
		*cl = CL[idx] + (CL[idx + 1] - CL[idx]) * d;
		*cd = 0.07 + oapiGetInducedDrag(*cl, 2.1, 0.65);
		// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff(double beta, double M, double Re, double* cl, double* cm, double* cd)
{
	const int nabsc = 8;
	int i;
	static const double BETA[nabsc] = { -180 * RAD,-135 * RAD,-90 * RAD,-45 * RAD,45 * RAD,90 * RAD,135 * RAD,180 * RAD };
	static const double CL[nabsc] = { 0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0 };
	for (i = 0; i < nabsc - 1 && BETA[i + 1] < beta; i++);
	*cl = CL[i] + (CL[i + 1] - CL[i]) * (beta - BETA[i]) / (BETA[i + 1] - BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag(*cl, 1.5, 0.6) + oapiGetWaveDrag(M, 0.75, 1.0, 1.1, 0.04);

	if (M > 1.0) *cd *= 1.9;
	else *cd *= 0.6;
}

// 2. vertical lift component (horizontal stabilisers and body)

void VLiftCoeff2(double aoa, double M, double Re, double* cl, double* cm, double* cd)
{
	const int nabsc = 8;
	int i;
	static const double AOA[nabsc] = { -180 * RAD,-135 * RAD,-90 * RAD,-45 * RAD,45 * RAD,90 * RAD,135 * RAD,180 * RAD };
	static const double CL[nabsc] = { 0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0 };
	for (i = 0; i < nabsc - 1 && AOA[i + 1] < aoa; i++);
	*cl = CL[i] + (CL[i + 1] - CL[i]) * (aoa - AOA[i]) / (AOA[i + 1] - AOA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag(*cl, 1.5, 0.6) + oapiGetWaveDrag(M, 0.75, 1.0, 1.1, 0.04);

	if (M > 1.0) *cd *= 1.9;
	else *cd *= 0.6;
}

Velcro::Velcro(OBJHANDLE hVessel, int flightmodel)
	: VESSEL3(hVessel, flightmodel)
{
	tex = oapiRegisterExhaustTexture("Exhaust_atsme");
	texstor = oapiRegisterExhaustTexture("RealExhaust");
	texkero = oapiRegisterExhaustTexture("KeroExhaust");
	texfluor = oapiRegisterExhaustTexture("FluorineExhaust");
	CAMERA_OFS = _V(-100, -100, -100);
	BecomeSomethingElse = false;
	init = false;
	is_multistage = false;
	EngineThrust = IspSL = IspVac = RCSThrust = Mass = MaxFuelMass = mayfly = 0;
	IspSL = -1;
	numexhausts = 0;
	numengines = 1;
	ph_first = NULL;
	Wing.Location = _V(0, 0, 0);
	Wing.span = -1.0;
	Wing.S = -1.0;
	Wing.AR = -1.0;
	HStab.Location = _V(0, 0, 0);
	HStab.span = -1.0;
	HStab.S = -1.0;
	HStab.AR = -1.0;
	VStab.Location = _V(0, 0, 0);
	VStab.span = -1.0;
	VStab.S = -1.0;
	VStab.AR = -1.0;
	padbias = 0;
	MEngineVec = _V(0, 0, 1);
	display_padbias_time = 0;
}

Velcro::~Velcro()
{
}


void Velcro::ScanLines(FILEHANDLE cfg, void* vs)
{
	int i = 0;
	char* pLine;
	while (oapiReadScenario_nextline(cfg, pLine))
	{
		if (!strnicmp(pLine, "Meshname", 8))
		{
			char* pLine2 = pLine + 8;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%31s", &meshname);
			}

		}
		else if (!strnicmp(pLine, "MaxMainThrust", 13))
		{
			char* pLine2 = pLine + 13;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &EngineThrust);
			}

		}
		else if (!strnicmp(pLine, "IspSL", 5))
		{
			char* pLine2 = pLine + 5;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &IspSL);
			}

		}
		else if (!strnicmp(pLine, "Isp", 3))
		{
			char* pLine2 = pLine + 3;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &IspVac);
			}

		}
		else if (!strnicmp(pLine, "MaxAttitudeThrust", 17))
		{
			char* pLine2 = pLine + 17;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &RCSThrust);
			}

		}
		else if (!strnicmp(pLine, "Mass", 4))
		{
			char* pLine2 = pLine + 4;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &Mass);
			}

		}
		else if (!strnicmp(pLine, "MaxFuel", 7))
		{
			char* pLine2 = pLine + 7;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &MaxFuelMass);
			}

		}
		else if (!strnicmp(pLine, "Multistage", 10))
		{
			char* pLine2 = pLine + 10;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				char value[31];
				pLine2++;
				sscanf(pLine2, "%31s", &value);
				if (!strnicmp(value, "true", 4))
					is_multistage = true;
			}

		}
		else if (!strnicmp(pLine, "PropellantResource1", 19))
		{
			char* pLine2 = pLine + 19;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &MaxFuelMass);
			}

		}
		else if (!strnicmp(pLine, "MEngineVec", 10))
		{
			char* pLine2 = pLine + 11;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf", &MEngineVec.x, &MEngineVec.y, &MEngineVec.z);
			}

		}
		else if (!strnicmp(pLine, "ExhaustType", 11))
		{
			char* pLine2 = pLine + 12;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				char ExhStr[31];
				pLine2++;
				sscanf(pLine2, "%31s", &ExhStr);
				numexhausts = 0;
				if (!strnicmp(ExhStr, "SRM", 3))
					ExhaustType = SRM;
				else if (!strnicmp(ExhStr, "N2O4", 4))
					ExhaustType = N2O4;
				else if (!strnicmp(ExhStr, "LF2", 3))
					ExhaustType = LF2;
				else if (!strnicmp(ExhStr, "LH2", 3))
					ExhaustType = LH2;
				else if (!strnicmp(ExhStr, "KERO", 4))
					ExhaustType = KERO;
				else if (!strnicmp(ExhStr, "HYBRID", 6))
					ExhaustType = HYBRID;
			}

		}
		else if (!strnicmp(pLine, "MEngineRef", 10))
		{
			char* pLine2 = pLine + 11;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				//				sscanf(pLine2, "%lf", &i);
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf", &MEngineRef[numexhausts].x, &MEngineRef[numexhausts].y, &MEngineRef[numexhausts].z);
				numexhausts++;
			}

		}
		else if (!strnicmp(pLine, "Wing", 4))
		{
			char* pLine2 = pLine + 4;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf%lf%lf%lf", &Wing.Location.x, &Wing.Location.y, &Wing.Location.z, &Wing.span, &Wing.S, &Wing.AR);
			}

		}
		else if (!strnicmp(pLine, "HStab", 5))
		{
			char* pLine2 = pLine + 5;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf%lf%lf%lf", &HStab.Location.x, &HStab.Location.y, &HStab.Location.z, &HStab.span, &HStab.S, &HStab.AR);
			}

		}
		else if (!strnicmp(pLine, "VStab", 5))
		{
			char* pLine2 = pLine + 5;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf%lf%lf%lf", &VStab.Location.x, &VStab.Location.y, &VStab.Location.z, &VStab.span, &VStab.S, &VStab.AR);
			}

		}
		else if (!strnicmp(pLine, "Become", 6))
		{
			char* pLine2 = pLine + 6;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%31s", &becomename);
				BecomeSomethingElse = true;
			}

		}
		else if (!strnicmp(pLine, "Mayfly", 6))
		{
			char* pLine2 = pLine + 6;
			while (*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if (*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &mayfly);
			}

		}
		else if (!strnicmp(pLine, "CAMERA", 6)) {
			sscanf(pLine + 6, "%lf%lf%lf", &CAMERA_OFS.x, &CAMERA_OFS.y, &CAMERA_OFS.z);
		}

		if (vs != NULL)
			ParseScenarioLineEx(pLine, vs);

	}
}

void Velcro::SetVelcroStage()
{
	ClearMeshes();
	if (Mass != -1.0)
		SetEmptyMass(Mass);
	SetPitchMomentScale(1e-6);
	SetBankMomentScale(1e-6);

	if (CAMERA_OFS.x != -100)
		SetCameraOffset(CAMERA_OFS);

	ClearAirfoilDefinitions();

	if (Wing.AR > 0.0)
		CreateAirfoil(LIFT_VERTICAL, Wing.Location, VLiftCoeff, Wing.span, Wing.S, Wing.AR);
	if (HStab.AR > 0.0)
		CreateAirfoil(LIFT_VERTICAL, HStab.Location, VLiftCoeff2, HStab.span, HStab.S, HStab.AR);
	if (VStab.AR > 0.0)
		CreateAirfoil(LIFT_HORIZONTAL, VStab.Location, HLiftCoeff, VStab.span, VStab.S, VStab.AR);

	VECTOR3 zero = _V(0, 0, 0);
	AddMesh(meshname, &zero);

	ClearPropellantResources();
	ph_first = CreatePropellantResource(MaxFuelMass, MaxFuelMass, 1.0);
	SetDefaultPropellantResource(ph_first);
	ClearThrusterDefinitions();

	if (IspSL < 0.0)
		IspSL = IspVac;

	th_first[0] = CreateThruster(_V(0, 0, MEngineRef[0].z), MEngineVec, EngineThrust, ph_first, IspVac, IspSL);
	thg_first = CreateThrusterGroup(th_first, 1, THGROUP_MAIN);
	numengines = 1;
	MakeExhausts();
	SetAttControlsVELCRO();
}

void Velcro::MakeExhausts() {

	//	th_first[0] = GetThrusterHandleByIndex(0);
	VECTOR3 dir = _V(-1 * MEngineVec.x, -1 * MEngineVec.y, -1 * MEngineVec.z);

	PARTICLESTREAMSPEC contrail =
	{
		0, pow(GetThrusterMax(th_first[0]) / (numexhausts + 1),0.3333) * 0.08, 24, 250, 0.4, 2.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, pow(GetThrusterMax(th_first[0]) / (numexhausts + 1),0.3333) * 0.04, 45, 250, 0.2, 0.4, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};
	exhaust_main.tex = oapiRegisterParticleTexture("Contrail2");


	if (numexhausts > 0)
	{
		ClearExhaustRefs();

		for (int i = 0; i < numexhausts; i++) {
			if (ExhaustType == N2O4)
				AddExhaust(th_first[0], pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.5, pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.035, MEngineRef[i], dir, texstor);
			else if (ExhaustType == LH2)
				AddExhaust(th_first[0], pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.5, pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.035, MEngineRef[i], dir, tex);
			else if (ExhaustType == KERO)
				AddExhaust(th_first[0], pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.75, pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.055, MEngineRef[i], dir, texkero);
			else if (ExhaustType == LF2)
				AddExhaust(th_first[0], pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.5, pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.035, MEngineRef[i], dir, texfluor);
			else if ((ExhaustType == SRM) || (ExhaustType == HYBRID)) {
				AddExhaust(th_first[0], pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.08, pow(GetThrusterMax(th_first[0]) / numexhausts, 0.3333) * 0.035, MEngineRef[i], dir, texstor);
				AddExhaustStream(th_first[0], MEngineRef[i], &contrail);
				AddExhaustStream(th_first[0], MEngineRef[i], &exhaust_main);
			}
		}
	}
}

void Velcro::SetAttControlsVELCRO()
{
	double dist = GetSize();

	THRUSTER_HANDLE th_rot[2];

	double isp = IspVac;
	if (is_multistage) {
		isp *= 100;
	}

	th_rot[0] = CreateThruster(_V(0, 0, dist), _V(0, 1, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(0, 0, -dist), _V(0, -1, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster(_V(0, 0, -dist), _V(0, 1, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(0, 0, dist), _V(0, 1, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster(_V(0, 0, -dist), _V(0, -1, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(0, 0, dist), _V(0, -1, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster(_V(0, 0, dist), _V(0, -1, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(0, 0, -dist), _V(0, 1, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster(_V(0, 0, dist), _V(-1, 0, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(0, 0, -dist), _V(1, 0, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster(_V(0, 0, dist), _V(1, 0, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(0, 0, -dist), _V(-1, 0, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster(_V(dist, 0.0, 0.0), _V(-1, 0, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster(_V(-dist, 0.0, 0.0), _V(1, 0, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster(_V(dist * 2, 0, 0), _V(0, -1, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(-dist * 2, 0, 0), _V(0, 1, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster(_V(dist * 2, 0, 0), _V(0, 1, 0), RCSThrust, ph_first, isp);
	th_rot[1] = CreateThruster(_V(-dist * 2, 0, 0), _V(0, -1, 0), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster(_V(0, 0, -dist), _V(0, 0, 1), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster(_V(0, 0, dist), _V(0, 0, -1), RCSThrust, ph_first, isp);
	CreateThrusterGroup(th_rot, 1, THGROUP_ATT_BACK);
}

void ParkVessel(VESSEL* ves)
{
	// Thanks to "jarmonik": https://www.orbiter-forum.com/threads/mars-2020-rover-and-mars-helicopter-scout.37153/page-6#post-577597
	// Also honorably mention to "face": https://www.orbiter-forum.com/threads/orbiter-economic-simulation.39629/page-2#post-578205
	OBJHANDLE ref = ves->GetSurfaceRef();
	double longitude, latitude, radius;
	ves->GetEquPos(longitude, latitude, radius);
	double heading = 0.0;
	OBJHANDLE obj = ves->GetHandle();
	oapiGetHeading(obj, &heading);
	VESSELSTATUS2 vs;
	memset(&vs, 0, sizeof(vs));
	vs.version = 2;
	vs.rbody = ref;
	vs.status = 1; // Landed
	vs.arot.x = 10; // <----- Undocumented feature "magic value" to land on touchdown points !! IMPORTANT !! It has to be 10, no more, no less !
	vs.surf_lng = longitude;
	vs.surf_lat = latitude;
	vs.surf_hdg = heading;
	ves->DefSetStateEx(&vs);
}

void Velcro::adjust_td() {
	
	if (!GroundContact()) {
		return;
	}

	VECTOR3 p4 = _V(0, 0, 1);
	
	const DWORD ntdvtx_geardown = 4;
	TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
		{_V(p1.x, p1.y, p1.z - padbias), 1e8, 1e6, 3, 1},
		{_V(p2.x, p2.y, p2.z - padbias), 1e8, 1e6, 3, 1},
		{_V(p3.x, p3.y, p3.z - padbias), 1e8, 1e6, 3, 1},
		{p4, 1e8, 1e6, 3},
	};
	SetTouchdownPoints(tdvtx_geardown, ntdvtx_geardown);
	ParkVessel(this);
}

int Velcro::clbkConsumeBufferedKey(DWORD key, bool down, char* kstate)
{
	if (!down) return 0; // only process keydown events
	switch (key) {
		case OAPI_KEY_EQUALS: {
			padbias += 1;
			adjust_td();
			display_padbias_time = oapiGetSimTime() + 3;
			break;
		}
		case OAPI_KEY_MINUS: {
			padbias -= 1;
			adjust_td();
			display_padbias_time = oapiGetSimTime() + 3;
			break;
		}
	}
	return 0;
}

void Velcro::clbkLoadStateEx(FILEHANDLE scn, void* vs)
{
	char* line;
	while (oapiReadScenario_nextline(scn, line)) {
		int p = 0;
		if (!_strnicmp(line, "PADBIAS", 7)) {
			sscanf(line + 7, "%d", &p);
			this->padbias = p;
		}
		else {
			ParseScenarioLineEx(line, vs);
		}
	}
}

void Velcro::clbkSaveState(FILEHANDLE scn)
{
	VESSEL2::clbkSaveState(scn);
	oapiWriteScenario_int(scn, "PADBIAS", (int)this->padbias);
}

void Velcro::clbkPreStep(double simt, double simdt, double mjd)
{
	if (!init) {
		GetTouchdownPoints(p1, p2, p3);
		adjust_td();
		init = true;
	}

	if (display_padbias_time > simt && oapiGetFocusInterface()->GetHandle() == GetHandle()) {
		sprintf(oapiDebugString(), "padbias %.2f", padbias);
	}
	else if (display_padbias_time != 0 && oapiGetFocusInterface()->GetHandle() == GetHandle()) {
		sprintf(oapiDebugString(), "");
		display_padbias_time = 0;
	}

}

void Velcro::clbkPostStep(double simt, double simdt, double mjd) {
}

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void Velcro::clbkSetClassCaps(FILEHANDLE cfg)
{
	ScanLines(cfg, NULL);
	SetVelcroStage();
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL* ovcInit(OBJHANDLE hvessel, int flightmodel)
{
	return new Velcro(hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit(VESSEL* vessel)
{
	if (vessel) delete (Velcro*)vessel;
}
